function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Nivo from '../../../Nivo';
import { marginPropType } from '../../../props';
import { getColorRange } from '../../../lib/colorUtils';
import { chord as d3Chord, ribbon as Ribbon } from 'd3-chord';
import { arc as Arc } from 'd3-shape';
import { rgb } from 'd3-color';

var Chord = function (_Component) {
    _inherits(Chord, _Component);

    function Chord() {
        _classCallCheck(this, Chord);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Chord.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            padAngle = _props.padAngle,
            innerRadiusRatio = _props.innerRadiusRatio,
            innerRadiusOffset = _props.innerRadiusOffset,
            ribbonOpacity = _props.ribbonOpacity,
            ribbonBorderWidth = _props.ribbonBorderWidth,
            arcOpacity = _props.arcOpacity,
            arcBorderWidth = _props.arcBorderWidth,
            colors = _props.colors;


        var color = getColorRange(colors);

        var margin = Object.assign({}, Nivo.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;
        var radius = Math.min(width, height) / 2;
        var arcInnerRadius = radius * innerRadiusRatio;
        var ribbonRadius = radius * (innerRadiusRatio - innerRadiusOffset);

        var chord = d3Chord().padAngle(padAngle);

        var arc = Arc().innerRadius(arcInnerRadius).outerRadius(radius);

        var ribbon = Ribbon().radius(ribbonRadius);

        var ribbons = chord(data);
        var arcs = ribbons.groups;

        return React.createElement(
            'svg',
            {
                xmlns: 'http://www.w3.org/2000/svg',
                className: 'nivo_chord',
                width: _width,
                height: _height
            },
            React.createElement(
                'g',
                {
                    className: 'nivo_chord_wrapper',
                    transform: 'translate(' + _width / 2 + ',' + _height / 2 + ')'
                },
                React.createElement(
                    'g',
                    { className: 'nivo_chord_ribbons' },
                    ribbons.map(function (d) {
                        var c = rgb(color(d.source.index));
                        c = rgb(c.r, c.g, c.b, ribbonOpacity);

                        return React.createElement('path', {
                            key: 'ribbon.' + d.source.index + '.' + d.target.index,
                            className: 'nivo_chord_ribbon',
                            d: ribbon(d),
                            fill: c,
                            stroke: c,
                            strokeWidth: ribbonBorderWidth
                        });
                    })
                ),
                React.createElement(
                    'g',
                    { className: 'nivo_chord_arcs' },
                    arcs.map(function (d) {
                        var c = rgb(color(d.index));
                        c = rgb(c.r, c.g, c.b, arcOpacity);

                        return React.createElement('path', {
                            key: 'arc.' + d.index,
                            className: 'nivo_chord_arc',
                            d: arc(d),
                            fill: c,
                            stroke: c,
                            strokeWidth: arcBorderWidth
                        });
                    })
                )
            )
        );
    };

    return Chord;
}(Component);

Chord.propTypes = {
    data: PropTypes.array.isRequired,
    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,

    padAngle: PropTypes.number.isRequired,
    innerRadiusRatio: PropTypes.number.isRequired,
    innerRadiusOffset: PropTypes.number.isRequired,

    ribbonOpacity: PropTypes.number.isRequired,
    ribbonBorderWidth: PropTypes.number.isRequired,

    // colors
    colors: PropTypes.any.isRequired
};

Chord.defaultProps = {
    // dimensions
    margin: Nivo.defaults.margin,

    padAngle: 0,
    innerRadiusRatio: 0.9,
    innerRadiusOffset: 0,

    ribbonOpacity: 0.5,
    ribbonBorderWidth: 1,

    arcOpacity: 1,
    arcBorderWidth: 1,

    // colors
    colors: Nivo.defaults.colorRange
};

export default Chord;