var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import { max, isEqual, merge } from 'lodash-es';
import React from 'react';
import PropTypes from 'prop-types';
import Nivo, { defaultTheme } from '../../../Nivo';
import { marginPropType, motionPropTypes, closedCurvePropType } from '../../../props';
import { getColorsGenerator } from '../../../lib/colorUtils';
import SvgWrapper from '../SvgWrapper';
import { scaleLinear } from 'd3-scale';
import RadarShapes from './RadarShapes';
import RadarGrid from './RadarGrid';
import RadarMarkers from './RadarMarkers';
import compose from 'recompose/compose';
import withPropsOnChange from 'recompose/withPropsOnChange';
import defaultProps from 'recompose/defaultProps';

var Radar = function Radar(_ref) {
    var facets = _ref.facets,
        data = _ref.data,
        curve = _ref.curve,
        radius = _ref.radius,
        radiusScale = _ref.radiusScale,
        angleStep = _ref.angleStep,
        centerX = _ref.centerX,
        centerY = _ref.centerY,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        borderWidth = _ref.borderWidth,
        borderColor = _ref.borderColor,
        gridLevels = _ref.gridLevels,
        gridShape = _ref.gridShape,
        gridLabelOffset = _ref.gridLabelOffset,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        enableMarkersLabel = _ref.enableMarkersLabel,
        markersLabel = _ref.markersLabel,
        markersLabelFormat = _ref.markersLabelFormat,
        markersLabelYOffset = _ref.markersLabelYOffset,
        theme = _ref.theme,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return React.createElement(
        SvgWrapper,
        { width: width, height: height, margin: margin },
        React.createElement(
            'g',
            { transform: 'translate(' + centerX + ', ' + centerY + ')' },
            React.createElement(RadarGrid, _extends({
                levels: gridLevels,
                shape: gridShape,
                radius: radius,
                angleStep: angleStep,
                theme: theme,
                facets: facets,
                labelOffset: gridLabelOffset
            }, motionProps)),
            React.createElement(RadarShapes, _extends({
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                curve: curve,
                borderWidth: borderWidth,
                borderColor: borderColor,
                fillOpacity: fillOpacity
            }, motionProps)),
            enableMarkers && React.createElement(RadarMarkers, _extends({
                facets: facets,
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                size: markersSize,
                color: markersColor,
                borderWidth: markersBorderWidth,
                borderColor: markersBorderColor,
                enableLabel: enableMarkersLabel,
                label: markersLabel,
                labelFormat: markersLabelFormat,
                labelYOffset: markersLabelYOffset,
                theme: theme
            }, motionProps))
        )
    );
};

Radar.propTypes = _extends({
    // data
    facets: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.number])).isRequired,
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        data: PropTypes.arrayOf(PropTypes.number).isRequired
    })).isRequired,

    curve: closedCurvePropType.isRequired,

    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,

    // border
    borderWidth: PropTypes.number.isRequired,
    borderColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),

    // grid
    gridLevels: PropTypes.number,
    gridShape: PropTypes.oneOf(['circular', 'linear']),
    gridLabelOffset: PropTypes.number,

    // markers
    enableMarkers: PropTypes.bool.isRequired,
    markersSize: PropTypes.number,
    markersColor: PropTypes.any,
    markersBorderWidth: PropTypes.number,
    markersBorderColor: PropTypes.any,
    enableMarkersLabel: PropTypes.bool,
    markersLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    markersLabelFormat: PropTypes.string,
    markersLabelYOffset: PropTypes.number,

    // theming
    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired,
    colorBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    fillOpacity: PropTypes.number.isRequired

}, motionPropTypes);

export var RadarDefaultProps = {
    // dimensions
    margin: Nivo.defaults.margin,

    curve: 'linearClosed',

    // border
    borderWidth: 2,
    borderColor: 'inherit',

    // grid
    gridLevels: 5,
    gridShape: 'circular',
    gridLabelOffset: 16,

    // markers
    enableMarkers: true,

    // theming
    theme: {},
    colors: 'nivo',
    colorBy: 'id',
    fillOpacity: 0.15,

    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};

var enhance = compose(defaultProps(RadarDefaultProps), withPropsOnChange(['theme'], function (props) {
    return { theme: merge({}, defaultTheme, props.theme) };
}), withPropsOnChange(['colors', 'colorBy'], function (_ref2) {
    var colors = _ref2.colors,
        colorBy = _ref2.colorBy;
    return {
        color: getColorsGenerator(colors, colorBy)
    };
}), withPropsOnChange(function (props, nextProps) {
    return props.facets !== nextProps.facets || props.data !== nextProps.data || props.width !== nextProps.width || props.color !== nextProps.color || !isEqual(props.margin, nextProps.margin);
}, function (_ref3) {
    var facets = _ref3.facets,
        data = _ref3.data,
        color = _ref3.color,
        _width = _ref3.width,
        _height = _ref3.height,
        _margin = _ref3.margin;

    var maxValue = max(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []));

    var margin = Object.assign({}, Nivo.defaults.margin, _margin);
    var width = _width - margin.left - margin.right;
    var height = _height - margin.top - margin.bottom;

    var radius = Math.min(width, height) / 2;
    var radiusScale = scaleLinear().range([0, radius]).domain([0, maxValue]);

    return {
        data: data.map(function (d) {
            return Object.assign({}, d, { color: color(d) });
        }),
        radius: radius,
        radiusScale: radiusScale,
        centerX: width / 2,
        centerY: height / 2,
        angleStep: Math.PI * 2 / facets.length
    };
}));

export default enhance(Radar);