'use strict';

exports.__esModule = true;
exports.LineDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _props = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _line = require('../../../lib/charts/line');

var _SmartMotion = require('../../SmartMotion');

var _SmartMotion2 = _interopRequireDefault(_SmartMotion);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _LineMarkers = require('./LineMarkers');

var _LineMarkers2 = _interopRequireDefault(_LineMarkers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Line = function Line(_ref) {
    var data = _ref.data,
        stacked = _ref.stacked,
        lineGenerator = _ref.lineGenerator,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        fullWidth = _ref.fullWidth,
        fullHeight = _ref.fullHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        theme = _ref.theme,
        color = _ref.color,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var lines = void 0;
    if (stacked === true) {
        lines = (0, _line.generateStackedLines)(data, xScale, yScale, color);
    } else {
        lines = (0, _line.generateLines)(data, xScale, yScale, color);
    }

    var markers = null;
    if (enableMarkers === true) {
        markers = _react2.default.createElement(_LineMarkers2.default, _extends({
            lines: lines,
            size: markersSize,
            color: (0, _colorUtils.getInheritedColorGenerator)(markersColor),
            borderWidth: markersBorderWidth,
            borderColor: (0, _colorUtils.getInheritedColorGenerator)(markersBorderColor)
        }, motionProps));
    }

    var lineNodes = void 0;
    if (animate === true) {
        var springConfig = {
            stiffness: motionStiffness,
            damping: motionDamping
        };

        lineNodes = lines.map(function (_ref2) {
            var id = _ref2.id,
                lineColor = _ref2.color,
                points = _ref2.points;
            return _react2.default.createElement(
                _SmartMotion2.default,
                {
                    key: id,
                    style: function style(spring) {
                        return {
                            d: spring(lineGenerator(points), springConfig),
                            stroke: spring(lineColor, springConfig)
                        };
                    }
                },
                function (style) {
                    return _react2.default.createElement('path', { key: id, d: style.d, fill: 'none', strokeWidth: 2, stroke: style.stroke });
                }
            );
        });
    } else {
        lineNodes = lines.map(function (_ref3) {
            var id = _ref3.id,
                lineColor = _ref3.color,
                points = _ref3.points;
            return _react2.default.createElement('path', {
                key: id,
                d: lineGenerator(points),
                fill: 'none',
                strokeWidth: 2,
                stroke: lineColor
            });
        });
    }

    return _react2.default.createElement(
        _SvgWrapper2.default,
        { width: fullWidth, height: fullHeight, margin: margin },
        _react2.default.createElement(_Grid2.default, _extends({
            theme: theme,
            width: width,
            height: height,
            xScale: enableGridX ? xScale : null,
            yScale: enableGridY ? yScale : null
        }, motionProps)),
        _react2.default.createElement(_Axes2.default, _extends({
            xScale: xScale,
            yScale: yScale,
            width: width,
            height: height,
            theme: theme,
            top: axisTop,
            right: axisRight,
            bottom: axisBottom,
            left: axisLeft
        }, motionProps)),
        lineNodes,
        markers
    );
};

Line.propTypes = _extends({
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
            x: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired,
            y: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired
        })).isRequired
    })).isRequired,

    stacked: _propTypes2.default.bool.isRequired,
    curve: _props.curvePropType.isRequired,
    lineGenerator: _propTypes2.default.func.isRequired,

    xScale: _propTypes2.default.func.isRequired,
    yScale: _propTypes2.default.func.isRequired,

    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _props.marginPropType,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // markers
    enableMarkers: _propTypes2.default.bool.isRequired,
    markersSize: _propTypes2.default.number.isRequired,
    markersColor: _propTypes2.default.any.isRequired,
    markersBorderWidth: _propTypes2.default.number.isRequired,
    markersBorderColor: _propTypes2.default.any.isRequired,

    // theming
    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired,
    colorBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    color: _propTypes2.default.func.isRequired

}, _props.motionPropTypes);

var LineDefaultProps = exports.LineDefaultProps = {
    stacked: false,
    curve: 'linear',

    // dimensions
    margin: _Nivo2.default.defaults.margin,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: true,

    // markers
    enableMarkers: true,
    markersSize: 6,
    markersColor: 'inherit',
    markersBorderWidth: 0,
    markersBorderColor: 'inherit',

    // theming
    theme: {},
    colors: _Nivo2.default.defaults.colorRange,
    colorBy: 'id',

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(LineDefaultProps), (0, _withPropsOnChange2.default)(['theme'], function (_ref4) {
    var theme = _ref4.theme;
    return { theme: (0, _lodash.merge)({}, _Nivo.defaultTheme, theme) };
}), (0, _withPropsOnChange2.default)(['curve'], function (_ref5) {
    var curve = _ref5.curve;
    return {
        lineGenerator: (0, _d3Shape.line)().x(function (d) {
            return d.x;
        }).y(function (d) {
            return d.y;
        }).curve(_props.curvePropMapping[curve])
    };
}), (0, _withPropsOnChange2.default)(['colors', 'colorBy'], function (_ref6) {
    var colors = _ref6.colors,
        colorBy = _ref6.colorBy;
    return {
        color: (0, _colorUtils.getColorsGenerator)(colors, colorBy)
    };
}), (0, _withPropsOnChange2.default)(function (props, nextProps) {
    return props.data !== nextProps.data || props.stacked !== nextProps.stacked || props.width !== nextProps.width || props.height !== nextProps.height || !(0, _lodash.isEqual)(props.margin, nextProps.margin);
}, function (_ref7) {
    var data = _ref7.data,
        stacked = _ref7.stacked,
        fullWidth = _ref7.width,
        fullHeight = _ref7.height,
        _margin = _ref7.margin;

    var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
    var width = fullWidth - margin.left - margin.right;
    var height = fullHeight - margin.top - margin.bottom;

    var scales = void 0;
    if (stacked === true) {
        scales = (0, _line.getStackedScales)(data, width, height);
    } else {
        scales = (0, _line.getScales)(data, width, height);
    }

    return _extends({
        margin: margin,
        width: width,
        height: height,
        fullWidth: fullWidth,
        fullHeight: fullHeight
    }, scales);
}), _pure2.default);

exports.default = enhance(Line);