'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _reactMotion = require('react-motion');

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _props2 = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _arcUtils = require('../../../lib/arcUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _d3Shape = require('d3-shape');

var _PieRadialLabels = require('./PieRadialLabels');

var _PieRadialLabels2 = _interopRequireDefault(_PieRadialLabels);

var _PieSlicesLabels = require('./PieSlicesLabels');

var _PieSlicesLabels2 = _interopRequireDefault(_PieSlicesLabels);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var Pie = function (_Component) {
    _inherits(Pie, _Component);

    function Pie() {
        _classCallCheck(this, Pie);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Pie.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            _innerRadius = _props.innerRadius,
            _padAngle = _props.padAngle,
            cornerRadius = _props.cornerRadius,
            borderWidth = _props.borderWidth,
            _borderColor = _props.borderColor,
            enableRadialLabels = _props.enableRadialLabels,
            radialLabel = _props.radialLabel,
            radialLabelsSkipAngle = _props.radialLabelsSkipAngle,
            radialLabelsLinkOffset = _props.radialLabelsLinkOffset,
            radialLabelsLinkDiagonalLength = _props.radialLabelsLinkDiagonalLength,
            radialLabelsLinkHorizontalLength = _props.radialLabelsLinkHorizontalLength,
            radialLabelsLinkStrokeWidth = _props.radialLabelsLinkStrokeWidth,
            radialLabelsTextXOffset = _props.radialLabelsTextXOffset,
            radialLabelsTextColor = _props.radialLabelsTextColor,
            radialLabelsLinkColor = _props.radialLabelsLinkColor,
            enableSlicesLabels = _props.enableSlicesLabels,
            sliceLabel = _props.sliceLabel,
            slicesLabelsSkipAngle = _props.slicesLabelsSkipAngle,
            slicesLabelsTextColor = _props.slicesLabelsTextColor,
            _theme = _props.theme,
            colors = _props.colors,
            colorBy = _props.colorBy,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;
        var centerX = width / 2;
        var centerY = height / 2;

        var padAngle = (0, _arcUtils.degreesToRadians)(_padAngle);

        var theme = (0, _lodash.merge)({}, _Nivo.defaultTheme, _theme);
        var color = (0, _colorUtils.getColorsGenerator)(colors, colorBy);
        var borderColor = (0, _colorUtils.getInheritedColorGenerator)(_borderColor);

        var motionProps = {
            animate: animate,
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        var radialLabelsProps = {
            label: (0, _propertiesConverters.getLabelGenerator)(radialLabel),
            skipAngle: radialLabelsSkipAngle,
            linkOffset: radialLabelsLinkOffset,
            linkDiagonalLength: radialLabelsLinkDiagonalLength,
            linkHorizontalLength: radialLabelsLinkHorizontalLength,
            linkStrokeWidth: radialLabelsLinkStrokeWidth,
            textXOffset: radialLabelsTextXOffset,
            textColor: (0, _colorUtils.getInheritedColorGenerator)(radialLabelsTextColor, 'axis.textColor'),
            linkColor: (0, _colorUtils.getInheritedColorGenerator)(radialLabelsLinkColor, 'axis.tickColor')
        };

        var slicesLabelsProps = {
            label: (0, _propertiesConverters.getLabelGenerator)(sliceLabel),
            skipAngle: slicesLabelsSkipAngle,
            textColor: (0, _colorUtils.getInheritedColorGenerator)(slicesLabelsTextColor, 'axis.textColor')
        };

        var radius = Math.min(width, height) / 2;
        var innerRadius = radius * Math.min(_innerRadius, 1);

        var pie = (0, _d3Shape.pie)();
        pie.value(function (d) {
            return d.value;
        });

        var arc = (0, _d3Shape.arc)();
        arc.outerRadius(radius);

        return _react2.default.createElement(
            _SvgWrapper2.default,
            { width: _width, height: _height, margin: margin },
            _react2.default.createElement(
                _reactMotion.Motion,
                {
                    style: {
                        centerX: (0, _reactMotion.spring)(centerX, motionProps),
                        centerY: (0, _reactMotion.spring)(centerY, motionProps),
                        innerRadius: (0, _reactMotion.spring)(innerRadius),
                        padAngle: (0, _reactMotion.spring)(padAngle, motionProps),
                        cornerRadius: (0, _reactMotion.spring)(cornerRadius, motionProps)
                    }
                },
                function (interpolatingStyle) {
                    var interpolatedPie = pie.padAngle(interpolatingStyle.padAngle);
                    var interpolatedArc = arc.cornerRadius(interpolatingStyle.cornerRadius).innerRadius(interpolatingStyle.innerRadius);

                    var arcsData = interpolatedPie(data).map(function (d) {
                        var angle = d.endAngle - d.startAngle;

                        return _extends({}, d, {
                            angle: angle,
                            angleDegrees: (0, _arcUtils.radiansToDegrees)(angle),
                            data: _extends({}, d.data, {
                                color: color(d.data)
                            })
                        });
                    });

                    return _react2.default.createElement(
                        'g',
                        {
                            transform: 'translate(' + interpolatingStyle.centerX + ', ' + interpolatingStyle.centerY + ')'
                        },
                        arcsData.map(function (d) {
                            return _react2.default.createElement('path', {
                                key: d.data.id,
                                d: interpolatedArc(d),
                                fill: d.data.color,
                                strokeWidth: borderWidth,
                                stroke: borderColor(d.data)
                            });
                        }),
                        enableSlicesLabels && _react2.default.createElement(_PieSlicesLabels2.default, _extends({
                            data: arcsData,
                            radius: radius,
                            innerRadius: interpolatingStyle.innerRadius,
                            theme: theme
                        }, slicesLabelsProps)),
                        enableRadialLabels && _react2.default.createElement(_PieRadialLabels2.default, _extends({
                            data: arcsData,
                            radius: radius,
                            theme: theme
                        }, radialLabelsProps))
                    );
                }
            )
        );
    };

    return Pie;
}(_react.Component);

Pie.propTypes = _extends({
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        value: _propTypes2.default.number.isRequired
    })).isRequired,

    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _props2.marginPropType,

    innerRadius: _propTypes2.default.number.isRequired,
    padAngle: _propTypes2.default.number.isRequired,
    cornerRadius: _propTypes2.default.number.isRequired,

    // border
    borderWidth: _propTypes2.default.number.isRequired,
    borderColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),

    // radial labels
    enableRadialLabels: _propTypes2.default.bool.isRequired,
    radialLabel: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    radialLabelsSkipAngle: _propTypes2.default.number,
    radialLabelsTextXOffset: _propTypes2.default.number,
    radialLabelsTextColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    radialLabelsLinkOffset: _propTypes2.default.number,
    radialLabelsLinkDiagonalLength: _propTypes2.default.number,
    radialLabelsLinkHorizontalLength: _propTypes2.default.number,
    radialLabelsLinkStrokeWidth: _propTypes2.default.number,
    radialLabelsLinkColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),

    // slices labels
    enableSlicesLabels: _propTypes2.default.bool.isRequired,
    sliceLabel: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    slicesLabelsSkipAngle: _propTypes2.default.number,
    slicesLabelsTextColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),

    // theming
    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired,
    colorBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func])

}, _props2.motionPropTypes);
Pie.defaultProps = {
    // dimensions
    margin: _Nivo2.default.defaults.margin,

    innerRadius: 0,
    padAngle: 0,
    cornerRadius: 0,

    // border
    borderWidth: 0,
    borderColor: 'inherit:darker(1)',

    // radial labels
    enableRadialLabels: true,
    radialLabel: 'id',
    radialLabelsTextColor: 'theme',
    radialLabelsLinkColor: 'theme',

    // slices labels
    enableSlicesLabels: true,
    sliceLabel: 'value',
    slicesLabelsTextColor: 'theme',

    // theming
    theme: {},
    colors: _Nivo2.default.defaults.colorRange,
    colorBy: 'id',

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};
exports.default = Pie;