'use strict';

exports.__esModule = true;
exports.RadarDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _lodash = require('lodash');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _props = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _d3Scale = require('d3-scale');

var _RadarShapes = require('./RadarShapes');

var _RadarShapes2 = _interopRequireDefault(_RadarShapes);

var _RadarGrid = require('./RadarGrid');

var _RadarGrid2 = _interopRequireDefault(_RadarGrid);

var _RadarMarkers = require('./RadarMarkers');

var _RadarMarkers2 = _interopRequireDefault(_RadarMarkers);

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Radar = function Radar(_ref) {
    var facets = _ref.facets,
        data = _ref.data,
        curve = _ref.curve,
        radius = _ref.radius,
        radiusScale = _ref.radiusScale,
        angleStep = _ref.angleStep,
        centerX = _ref.centerX,
        centerY = _ref.centerY,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        borderWidth = _ref.borderWidth,
        borderColor = _ref.borderColor,
        gridLevels = _ref.gridLevels,
        gridShape = _ref.gridShape,
        gridLabelOffset = _ref.gridLabelOffset,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        theme = _ref.theme,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return _react2.default.createElement(
        _SvgWrapper2.default,
        { width: width, height: height, margin: margin },
        _react2.default.createElement(
            'g',
            { transform: 'translate(' + centerX + ', ' + centerY + ')' },
            _react2.default.createElement(_RadarGrid2.default, _extends({
                levels: gridLevels,
                shape: gridShape,
                radius: radius,
                angleStep: angleStep,
                theme: theme,
                facets: facets,
                labelOffset: gridLabelOffset
            }, motionProps)),
            _react2.default.createElement(_RadarShapes2.default, _extends({
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                curve: curve,
                borderWidth: borderWidth,
                borderColor: borderColor,
                fillOpacity: fillOpacity
            }, motionProps)),
            enableMarkers && _react2.default.createElement(_RadarMarkers2.default, _extends({
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                size: markersSize,
                color: markersColor,
                borderWidth: markersBorderWidth,
                borderColor: markersBorderColor
            }, motionProps))
        )
    );
};

Radar.propTypes = _extends({
    // data
    facets: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])).isRequired,
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.number).isRequired
    })).isRequired,

    curve: _props.closedCurvePropType.isRequired,

    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _props.marginPropType,

    // border
    borderWidth: _propTypes2.default.number.isRequired,
    borderColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),

    // grid
    gridLevels: _propTypes2.default.number,
    gridShape: _propTypes2.default.oneOf(['circular', 'linear']),
    gridLabelOffset: _propTypes2.default.number,

    // markers
    enableMarkers: _propTypes2.default.bool.isRequired,
    markersSize: _propTypes2.default.number,
    markersColor: _propTypes2.default.any,
    markersBorderWidth: _propTypes2.default.number,
    markersBorderColor: _propTypes2.default.any,

    // theming
    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired,
    colorBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    fillOpacity: _propTypes2.default.number.isRequired

}, _props.motionPropTypes);

var RadarDefaultProps = exports.RadarDefaultProps = {
    // dimensions
    margin: _Nivo2.default.defaults.margin,

    curve: 'linearClosed',

    // border
    borderWidth: 2,
    borderColor: 'inherit',

    // grid
    gridLevels: 5,
    gridShape: 'circular',
    gridLabelOffset: 16,

    // markers
    enableMarkers: true,

    // theming
    theme: {},
    colors: _Nivo2.default.defaults.colorRange,
    colorBy: 'id',
    fillOpacity: 0.15,

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(RadarDefaultProps), (0, _withPropsOnChange2.default)(['theme'], function (props) {
    return { theme: (0, _lodash.merge)({}, _Nivo.defaultTheme, props.theme) };
}), (0, _withPropsOnChange2.default)(['colors', 'colorBy'], function (_ref2) {
    var colors = _ref2.colors,
        colorBy = _ref2.colorBy;
    return {
        color: (0, _colorUtils.getColorsGenerator)(colors, colorBy)
    };
}), (0, _withPropsOnChange2.default)(function (props, nextProps) {
    return props.facets !== nextProps.facets || props.data !== nextProps.data || props.width !== nextProps.width || props.color !== nextProps.color || !(0, _lodash.isEqual)(props.margin, nextProps.margin);
}, function (_ref3) {
    var facets = _ref3.facets,
        data = _ref3.data,
        color = _ref3.color,
        _width = _ref3.width,
        _height = _ref3.height,
        _margin = _ref3.margin;

    var maxValue = (0, _lodash.max)(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []));

    var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
    var width = _width - margin.left - margin.right;
    var height = _height - margin.top - margin.bottom;

    var radius = Math.min(width, height) / 2;
    var radiusScale = (0, _d3Scale.scaleLinear)().range([0, radius]).domain([0, maxValue]);

    return {
        data: data.map(function (d) {
            return Object.assign({}, d, { color: color(d) });
        }),
        radius: radius,
        radiusScale: radiusScale,
        centerX: width / 2,
        centerY: height / 2,
        angleStep: Math.PI * 2 / facets.length
    };
}));

exports.default = enhance(Radar);